import unittest
import os
from xml_archive_to_pdf.utils import get_bare_tag, get_clean_text, get_doc, \
registerFont, get_label, has_children, add_logo, ptTomm, write_table, write_elem, \
is_writable_element, calcul_level, is_leaving_table, build_pdf, is_started_table
import xml.etree.cElementTree as ET
from reportlab.pdfbase import pdfmetrics
from reportlab.platypus.flowables import Image
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, Image, Table, Indenter
from xml_archive_to_pdf.styles import get_styles

"""
Utils unit tests
"""


class UtilsTest(unittest.TestCase):

    def setUp(self):
        self.xml_file = "tests/data/pathfinder_1.xml"
        self.tree = ET.parse(self.xml_file)
        self.pdf_file = "tests/data/pathfinder_1_tmp.pdf"
        self.font_folder = "tests/data/CustomFont"
        self.logo_file = "tests/data/logo.png"

    def test_build_pdf(self):
        build_pdf(self.xml_file, self.pdf_file, self.logo_file, self.font_folder)
        self.assertTrue(os.path.exists(self.pdf_file))
        self.assertTrue(os.path.getsize(self.pdf_file) > 0)

    def test_get_bare_tag(self):
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}element")
        bare_tag = get_bare_tag(elem)
        self.assertEqual(bare_tag, "element")

    def test_get_clean_text(self):
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}qualite")
        clean_text = get_clean_text(elem)
        self.assertEqual(clean_text, "très bonne")

    def test_get_doc(self):
        doc = get_doc(self.pdf_file)
        self.assertEqual(doc.pagesize, (612.0, 792.0))
        self.assertEqual(doc.bottomMargin, 12*2)
        self.assertEqual(doc.topMargin, 12*2)
        self.assertEqual(doc.leftMargin, 12*2)
        self.assertEqual(doc.rightMargin, 12*2)

    def test_registerFont(self):
        registerFont("CustomFont", self.font_folder)
        self.assertIsNotNone(pdfmetrics.getFont("CustomFont"))

    def test_get_label(self):
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}element")
        self.assertEqual(get_label(elem), "élément")
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}niveau")
        self.assertEqual(get_label(elem), "niveau")
        elem = self.tree.findall(".//{fr:unistra:di:archive:pathfinder:v1}classe")[1]
        self.assertEqual(get_label(elem), "")

    def test_has_children(self):
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}element")
        self.assertFalse(has_children(elem))
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}classes")
        self.assertTrue(has_children(elem))

    def test_add_logo(self):
        Story = []
        add_logo(Story, self.logo_file)
        self.assertTrue(isinstance(Story[0], Image))
        self.assertTrue(isinstance(Story[1], Spacer))

    def test_ptTomm(self):
        r = ptTomm(3)
        self.assertEqual(r, 1.0583339999999999)

    def test_write_table(self):
        Story = []
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}armes")
        write_table(Story, elem, 2, get_styles())
        self.assertEqual(len(Story), 3)
        # 0
        self.assertEqual(Story[0].style.name, "cHeading2")
        self.assertEqual(Story[0].text, "Liste des armes")
        # 1
        self.assertTrue(isinstance(Story[1], Spacer))
        # 2
        self.assertTrue(isinstance(Story[2], Table))
        # table
        self.assertEqual(Story[2]._colWidths,
            [67.99611118110236, 67.99611118110236, 67.99611118110236, 67.99611118110236,
             67.99611118110236, 67.99611118110236, 67.99611118110236, 67.99611118110236])
        self.assertEqual(len(Story[2].__dict__["_cellvalues"]), 4)
        self.assertEqual(len(Story[2].__dict__["_cellvalues"][0]), 8)
        self.assertEqual(len(Story[2].__dict__["_cellvalues"][1]), 8)
        self.assertEqual(len(Story[2].__dict__["_cellvalues"][2]), 8)
        self.assertEqual(len(Story[2].__dict__["_cellvalues"][3]), 8)
        # Column titles
        self.assertEqual(Story[2].__dict__["_cellvalues"][0][0].text, "<b>nom</b>")
        self.assertEqual(Story[2].__dict__["_cellvalues"][0][1].text, "<b>type</b>")
        self.assertEqual(Story[2].__dict__["_cellvalues"][0][2].text, "<b>portée</b>")
        self.assertEqual(Story[2].__dict__["_cellvalues"][0][3].text, "<b>dégât</b>")
        self.assertEqual(Story[2].__dict__["_cellvalues"][0][4].text, "<b>élément</b>")
        self.assertEqual(Story[2].__dict__["_cellvalues"][0][5].text, "<b>rareté</b>")
        self.assertEqual(Story[2].__dict__["_cellvalues"][0][6].text, "<b>prix</b>")
        self.assertEqual(Story[2].__dict__["_cellvalues"][0][7].text, "<b>qualité</b>")
        # First row
        self.assertEqual(Story[2].__dict__["_cellvalues"][1][0].text, "cimeterre")
        self.assertEqual(Story[2].__dict__["_cellvalues"][1][1].text, "à deux mains")
        self.assertEqual(Story[2].__dict__["_cellvalues"][1][2].text, "3")
        self.assertEqual(Story[2].__dict__["_cellvalues"][1][3].text, "7")
        self.assertEqual(Story[2].__dict__["_cellvalues"][1][4].text, "feu")
        self.assertEqual(Story[2].__dict__["_cellvalues"][1][5].text, "épique")
        self.assertEqual(Story[2].__dict__["_cellvalues"][1][6].text, "1000")
        self.assertEqual(Story[2].__dict__["_cellvalues"][1][7].text, "très bonne")
        # Second row
        self.assertEqual(Story[2].__dict__["_cellvalues"][2][0].text, "arc")
        self.assertEqual(Story[2].__dict__["_cellvalues"][2][1].text, "à distance")
        self.assertEqual(Story[2].__dict__["_cellvalues"][2][2].text, "8")
        self.assertEqual(Story[2].__dict__["_cellvalues"][2][3].text, "2")
        self.assertEqual(Story[2].__dict__["_cellvalues"][2][4].text, "glace")
        self.assertEqual(Story[2].__dict__["_cellvalues"][2][5].text, "simple")
        self.assertEqual(Story[2].__dict__["_cellvalues"][2][6].text, "100")
        self.assertEqual(Story[2].__dict__["_cellvalues"][2][7].text, "mauvaise")
        # Thrid row
        self.assertEqual(Story[2].__dict__["_cellvalues"][3][0].text, "épée")
        self.assertEqual(Story[2].__dict__["_cellvalues"][3][1].text, "à une main")
        self.assertEqual(Story[2].__dict__["_cellvalues"][3][2].text, "5")
        self.assertEqual(Story[2].__dict__["_cellvalues"][3][3].text, "3")
        self.assertEqual(Story[2].__dict__["_cellvalues"][3][4].text, "terre")
        self.assertEqual(Story[2].__dict__["_cellvalues"][3][5].text, "rare")
        self.assertEqual(Story[2].__dict__["_cellvalues"][3][6].text, "500")
        self.assertEqual(Story[2].__dict__["_cellvalues"][3][7].text, "moyenne")

    def test_write_elem(self):
        Story = []
        # title
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}etat-civil")
        write_elem(Story, elem, 1, get_styles())
        self.assertEqual(len(Story), 1)
        self.assertEqual(Story[0].text, "son état civil")
        self.assertEqual(Story[0].style.name, "cHeading1")
        # key value
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}age")
        write_elem(Story, elem, 2, get_styles(), 2)
        self.assertEqual(len(Story), 2)
        self.assertEqual(Story[1].text, "<b>son âge :</b> 20")
        self.assertEqual(Story[1].style.name, "cNormal2")
        # forced title
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}quetes")
        write_elem(Story, elem, 1, get_styles())
        self.assertEqual(len(Story), 3)
        self.assertEqual(Story[2].text, "Quêtes en cours")
        self.assertEqual(Story[2].style.name, "cHeading1")

    def test_is_writable_element(self):
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}etat-civil")
        r = is_writable_element(elem, "start", (False, 0))
        self.assertTrue(r)
        r = is_writable_element(elem, "start", (True, 1))
        self.assertFalse(r)
        r = is_writable_element(elem, "end", (False, 0))
        self.assertFalse(r)

    def test_is_started_tablet(self):
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}armes")
        r = is_started_table(elem, "start", (False, 0))
        self.assertTrue(r)
        r = is_started_table(elem, "start", (True, 1))
        self.assertFalse(r)
        r = is_started_table(elem, "end", (False, 0))
        self.assertFalse(r)
        elem = self.tree.find(".//{fr:unistra:di:archive:pathfinder:v1}etat-civil")
        r = is_started_table(elem, "start", (False, 0))
        self.assertFalse(r)
        r = is_started_table(elem, "start", (True, 1))
        self.assertFalse(r)
        r = is_started_table(elem, "end", (False, 0))
        self.assertFalse(r)

    def test_calcul_level(self):
        self.assertEqual(4, calcul_level("start", 3))
        self.assertEqual(5, calcul_level("end", 6))

    def test_is_leaving_table(self):
        self.assertFalse(is_leaving_table("start", 3, (False, 0)))
        self.assertFalse(is_leaving_table("start", 3, (True, 2)))
        self.assertFalse(is_leaving_table("start", 3, (True, 3)))
        self.assertFalse(is_leaving_table("end", 3, (False, 0)))
        self.assertFalse(is_leaving_table("end", 3, (True, 2)))
        self.assertTrue(is_leaving_table("end", 3, (True, 3)))

    def tearDown(self):
        if os.path.exists(self.pdf_file):
            os.remove(self.pdf_file)



import unittest
from xml_archive_to_pdf.styles import get_styles, get_title_style, get_normal_style, get_table_style
from xml_archive_to_pdf.utils import registerFont
import xml.etree.cElementTree as ET
from reportlab.platypus import TableStyle

"""
Styles unit tests
"""


class StylesTest(unittest.TestCase):

    def setUp(self):
        self.xml_file = "tests/data/pathfinder_1.xml"
        self.tree = ET.parse(self.xml_file)
        self.pdf_file = "tests/data/pathfinder_1_tmp.pdf"
        self.font_folder = "tests/data/CustomFont"
        self.logo_file = "tests/data/logo.png"
        self.styles = get_styles()

    def test_get_styles_without_font(self):
            styles = self.styles
            self.assertIsNotNone(styles["cTitle"])
            self.assertEqual(styles["cTitle"].fontName, "Helvetica-Bold")
            self.assertEqual(0, styles["cTitle"].leftIndent)
            self.assertIsNotNone(styles["cHeading1"])
            self.assertEqual(styles["cHeading1"].fontName, "Helvetica-Bold")
            self.assertEqual(0, styles["cHeading1"].leftIndent)
            self.assertIsNotNone(styles["cHeading2"])
            self.assertEqual(styles["cHeading2"].fontName, "Helvetica-Bold")
            self.assertEqual(12, styles["cHeading2"].leftIndent)
            self.assertIsNotNone(styles["cHeading3"])
            self.assertEqual(styles["cHeading3"].fontName, "Helvetica-BoldOblique")
            self.assertEqual(24, styles["cHeading3"].leftIndent)
            self.assertIsNotNone(styles["cHeading4"])
            self.assertEqual(styles["cHeading4"].fontName, "Helvetica-BoldOblique")
            self.assertEqual(36, styles["cHeading4"].leftIndent)
            self.assertIsNotNone(styles["cHeading5"])
            self.assertEqual(styles["cHeading5"].fontName, "Helvetica-Bold")
            self.assertEqual(48, styles["cHeading5"].leftIndent)
            self.assertIsNotNone(styles["cHeading6"])
            self.assertEqual(styles["cHeading6"].fontName, "Helvetica-Bold")
            self.assertEqual(60, styles["cHeading6"].leftIndent)
            self.assertIsNotNone(styles["cNormal"])
            self.assertEqual(styles["cNormal"].fontName, "Helvetica")
            self.assertEqual(0, styles["cNormal"].leftIndent)
            self.assertIsNotNone(styles["cNormal1"])
            self.assertEqual(styles["cNormal1"].fontName, "Helvetica")
            self.assertEqual(12, styles["cNormal1"].leftIndent)
            self.assertIsNotNone(styles["cNormal2"])
            self.assertEqual(styles["cNormal2"].fontName, "Helvetica")
            self.assertEqual(24, styles["cNormal2"].leftIndent)
            self.assertIsNotNone(styles["cNormal3"])
            self.assertEqual(styles["cNormal3"].fontName, "Helvetica")
            self.assertEqual(36, styles["cNormal3"].leftIndent)
            self.assertIsNotNone(styles["cNormal4"])
            self.assertEqual(styles["cNormal4"].fontName, "Helvetica")
            self.assertEqual(48, styles["cNormal4"].leftIndent)
            self.assertIsNotNone(styles["cNormal5"])
            self.assertEqual(styles["cNormal5"].fontName, "Helvetica")
            self.assertEqual(60, styles["cNormal5"].leftIndent)
            self.assertIsNotNone(styles["cNormal6"])
            self.assertEqual(styles["cNormal6"].fontName, "Helvetica")
            self.assertEqual(72, styles["cNormal6"].leftIndent)
            self.assertIsNotNone(styles["cBodyText"])
            self.assertEqual(styles["cBodyText"].fontName, "Helvetica")
            self.assertEqual(0, styles["cBodyText"].leftIndent)
            self.assertIsNotNone(styles["cItalic"])
            self.assertEqual(styles["cItalic"].fontName, "Helvetica-Oblique")
            self.assertEqual(0, styles["cItalic"].leftIndent)

    def test_get_styles_with_font(self):
        if self.font_folder:
            registerFont("CustomFont", self.font_folder)
            styles = get_styles("CustomFont")
            self.assertIsNotNone(styles["cTitle"])
            self.assertEqual(styles["cTitle"].fontName, "CustomFont-Bold")
            self.assertEqual(0, styles["cTitle"].leftIndent)
            self.assertIsNotNone(styles["cHeading1"])
            self.assertEqual(styles["cHeading1"].fontName, "CustomFont-Bold")
            self.assertEqual(0, styles["cHeading1"].leftIndent)
            self.assertIsNotNone(styles["cHeading2"])
            self.assertEqual(styles["cHeading2"].fontName, "CustomFont-Bold")
            self.assertEqual(12, styles["cHeading2"].leftIndent)
            self.assertIsNotNone(styles["cHeading3"])
            self.assertEqual(styles["cHeading3"].fontName, "CustomFont-BoldOblique")
            self.assertEqual(24, styles["cHeading3"].leftIndent)
            self.assertIsNotNone(styles["cHeading4"])
            self.assertEqual(styles["cHeading4"].fontName, "CustomFont-BoldOblique")
            self.assertEqual(36, styles["cHeading4"].leftIndent)
            self.assertIsNotNone(styles["cHeading5"])
            self.assertEqual(styles["cHeading5"].fontName, "CustomFont-Bold")
            self.assertEqual(48, styles["cHeading5"].leftIndent)
            self.assertIsNotNone(styles["cHeading6"])
            self.assertEqual(styles["cHeading6"].fontName, "CustomFont-Bold")
            self.assertEqual(60, styles["cHeading6"].leftIndent)
            self.assertIsNotNone(styles["cNormal"])
            self.assertEqual(styles["cNormal"].fontName, "CustomFont")
            self.assertEqual(0, styles["cNormal"].leftIndent)
            self.assertIsNotNone(styles["cNormal1"])
            self.assertEqual(styles["cNormal1"].fontName, "CustomFont")
            self.assertEqual(12, styles["cNormal1"].leftIndent)
            self.assertIsNotNone(styles["cNormal2"])
            self.assertEqual(styles["cNormal2"].fontName, "CustomFont")
            self.assertEqual(24, styles["cNormal2"].leftIndent)
            self.assertIsNotNone(styles["cNormal3"])
            self.assertEqual(styles["cNormal3"].fontName, "CustomFont")
            self.assertEqual(36, styles["cNormal3"].leftIndent)
            self.assertIsNotNone(styles["cNormal4"])
            self.assertEqual(styles["cNormal4"].fontName, "CustomFont")
            self.assertEqual(48, styles["cNormal4"].leftIndent)
            self.assertIsNotNone(styles["cNormal5"])
            self.assertEqual(styles["cNormal5"].fontName, "CustomFont")
            self.assertEqual(60, styles["cNormal5"].leftIndent)
            self.assertIsNotNone(styles["cNormal6"])
            self.assertEqual(styles["cNormal6"].fontName, "CustomFont")
            self.assertEqual(72, styles["cNormal6"].leftIndent)
            self.assertIsNotNone(styles["cBodyText"])
            self.assertEqual(styles["cBodyText"].fontName, "CustomFont")
            self.assertEqual(0, styles["cBodyText"].leftIndent)
            self.assertIsNotNone(styles["cItalic"])
            self.assertEqual(styles["cItalic"].fontName, "CustomFont-Oblique")
            self.assertEqual(0, styles["cItalic"].leftIndent)

    def test_get_title_style(self):
        self.assertEqual(get_title_style(self.styles, 3).name, "cHeading3")
        self.assertEqual(get_title_style(self.styles, 50).name, "cItalic")
        self.assertEqual(get_title_style(self.styles, -1).name, "cItalic")

    def test_get_normal_style(self):
        self.assertEqual(get_normal_style(self.styles, 3).name, "cNormal3")
        self.assertEqual(get_normal_style(self.styles, 50).name, "cNormal6")
        self.assertEqual(get_normal_style(self.styles, -1).name, "cNormal6")

    def test_get_table_style(self):
        ts = get_table_style()
        self.assertTrue(isinstance(ts, TableStyle))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
from setuptools import find_packages

with open('README.rst') as readme:
    long_description = readme.read()

with open('requirements.txt') as requirements:
    lines = requirements.readlines()
    libraries = [lib for lib in lines if not lib.startswith('-')]
    dependency_links = [link.split()[1] for link in lines if
                        link.startswith('-f')]

setup(
    name='xml-archive-to-pdf',
    version='1.0.9',
    author='di-dip-unistra',
    author_email='di-dip@unistra.fr',
    maintainer='di-dip-unistra',
    maintainer_email='di-dip@unistra.fr',
    url='https://github.com/unistra/xml-archive-to-pdf',
    license='PSF',
    description="Transformation d'un fichier xml de type unistra:archive en fichier pdf",
    long_description=long_description,
    packages=find_packages(),
    download_url='https://github.com/unistra/xml-archive-to-pdf',
    install_requires=libraries,
    dependency_links=dependency_links,
    keywords=[],
    entry_points={
        'console_scripts': [
            'xml-archive-to-pdf = xml_archive_to_pdf.main:main',
        ],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.4"
    ]
)



"""
xml-archive-to-pdf

Usage:
    xml-archive-to-pdf (-i <xml_file>) (-o <pdf_file>)
    xml-archive-to-pdf (-i <xml_file>) (-o <pdf_file>) -l <logo_file>
    xml-archive-to-pdf (-i <xml_file>) (-o <pdf_file>) -f <font_folder>
    xml-archive-to-pdf (-i <xml_file>) (-o <pdf_file>) -l <logo_file> -f <font_folder>

Options:
    -h --help  aide
    -i <xml_file>, --input <xml_file>
    -o <pdf_file>, --output <pdf_file>
    -l <logo_file>, --logo <logo_file>
    -f <font_folder>, --font <font_folder>
"""

from docopt import docopt
from xml_archive_to_pdf.utils import build_pdf
import sys


def main(args=None):
    try:
        if not args:
            args = docopt(__doc__)
        xml_file = args.get("--input")
        pdf_file = args.get("--output")
        logo_file = args.get("--logo")
        font_folder = args.get("--font")
        build_pdf(xml_file, pdf_file, logo_file, font_folder)
    except Exception as e:
        sys.stderr.write(str(e))
        sys.exit(1)

if __name__ == "__main__":
    main()
